
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/xv/xvCore.h>
#include <X11/xv/xvBasicBitmap.h>
#include <X11/xv/xvFont.h>
#include <X11/xv/xvForm.h>
#include <X11/xv/xvLabel.h>
#include <X11/xv/xvButton.h>
#include <X11/xv/xvPopupShell.h>
#include <X11/xv/xvView.h>
#include <X11/xv/xvList.h>

#include <X11/xv/xvDropDown.h>

#include <X11/xv/downptr>
#include <stdio.h>

// ------------------------------------

xvDropDown::xvDropDown()
{
}

// ------------------------------------

xvDropDown::xvDropDown(String name, xvCore & parent, int numargs, char **argdata,
				XtCallbackProc button_callback, XtCallbackProc select_callback,
				int vdepth, int currentindex)
{
	init(name, parent, numargs, argdata, button_callback, select_callback, vdepth, currentindex);
}

// ------------------------------------

xvDropDown::~xvDropDown()
{	
	delete copydata;
}

// ------------------------------------

Widget xvDropDown::ID()
{
	return(dropform.ID());
}

// ------------------------------------

void xvDropDown::init(String name, xvCore & parent, int numargs, char **argdata,
				XtCallbackProc button_callback, XtCallbackProc select_callback,
				int vdepth, int currentindex)
{
	
	current=currentindex;
	numlist=numargs;
	ddepth=vdepth;
	
	copydata= new char*[numargs];
	dropform.init(name, parent, FALSE, TRUE);
	dropform.SetDefaultDistance(1);
	w=dropform.ID();

	droplabel.init("droplabel", dropform);
	droplabel.SetLabel(argdata[current]);
	droplabel.SetJustify(XtJustifyLeft);
	droplabel.SetFixedPosition(TRUE);
	droplabel.SetBorderWidth(0);
	droplabel.SetShadowWidth(3);
	droplabel.SetBeNiceToColormap(FALSE);
			
	dropbutton.init("dropbutton", dropform,  button_callback);
	dropbutton.SetRelHorizontal(droplabel);
	dropbutton.SetBorderWidth(0);
	dropbutton.SetShadowWidth(3);
	dropbutton.SetLabel(NULL);
	dropbutton.SetBitmapInfo(downptr_bits, downptr_width, downptr_height, XtNbitmap);
	dropbutton.SetFixedPosition(TRUE);
	dropbutton.SetBeNiceToColormap(FALSE);

	dropshell.init("dropshell", droplabel);
	dropview.init("dropview", dropshell, FALSE, FALSE, TRUE);

	droplist.init("droplist", dropview, select_callback, numargs, argdata, 1);
	droplist.Highlight(current);
	dropview.ManageChildren(TRUE);

	labelwidth=droplist.GetLongest()+20;

	CalcHeights();

	for (int j=0; j<numargs; j++)
		copydata[j]=argdata[j];	
	
	dropform.ManageChildren(TRUE);
	bwidth=dropbutton.GetWidth();

}

// -------------------------------------

void xvDropDown::ChangeWidth(Dimension wid)
{
	int lw=wid-(bwidth+7);
	dropbutton.SetWidth(bwidth);
	labelwidth=lw;
	CalcHeights();
}

// -------------------------------------

void xvDropDown::ChangeFont(String name)
{
	if (name != NULL) {
		droplabel.ChangeFont(name);
		droplist.ChangeFont(name);
	}
}

// -------------------------------------

void xvDropDown::DropDownOpen()
{
	dropshell.Open();
}

// ------------------------------------

int xvDropDown::DropDownClose(XtPointer calldata)
{
	XawListReturnStruct *lr = (XawListReturnStruct *) calldata;
	current=lr->list_index;
	droplabel.SetLabel(copydata[current]);
	dropshell.Close();
	return(current);
}

// ------------------------------------

void xvDropDown::CalcHeights()
{
	Dimension ht=dropbutton.GetHeight();
	if (droplabel.GetHeight() > ht)
		ht=droplabel.GetHeight();

	droplabel.SetHeight(ht);
	dropbutton.SetHeight(ht);
	droplabel.SetWidth(labelwidth);
	droplist.SetWidth(labelwidth);

	int listh=droplist.GetHeight()/numlist;
		
	if (numlist > ddepth)
		dropview.SetHeight(ddepth*listh);		
}

// ------------------------------------

void xvDropDown::SetCurrent(int xcur)
{
	current=xcur;
	droplabel.SetLabel(copydata[current]);
	droplist.Highlight(current);
	CalcHeights();
}

// ------------------------------------

void xvDropDown::SetButtonResource(String rname, XtArgVal resource)
{
	XtVaSetValues(dropbutton.ID(), rname, resource, NULL);
}

// ------------------------------------

void xvDropDown::SetListResource(String rname, XtArgVal resource)
{
	XtVaSetValues(droplist.ID(), rname, resource, NULL);
}

// ------------------------------------

void xvDropDown::SetLabelResource(String rname, XtArgVal resource)
{
	XtVaSetValues(droplabel.ID(), rname, resource, NULL);
}

// ------------------------------------

void xvDropDown::SetAllResource(String rname, XtArgVal resource)
{
	SetLabelResource(rname, resource);
	SetButtonResource(rname, resource);
	SetListResource(rname, resource);
}

// ------------------------------------

void xvDropDown::SetBackground(String name)
{
	dropform.SetBackground(name);
	droplabel.SetBackground(name);
	dropbutton.SetBackground(name);
	droplist.SetBackground(name);
}

// ------------------------------------

void xvDropDown::SetForeground(String name)
{
	droplabel.SetForeground(name);
	dropbutton.SetForeground(name);
	droplist.SetForeground(name);
}

// ------------------------------------

void xvDropDown::SetSensitive(Boolean sense)
{
	droplabel.SetSensitive(sense);
	dropbutton.SetSensitive(sense);
}

// ------------------------------------

void xvDropDown::SetNewList(int numargs, char **argdata, int currentindex)
{
	delete copydata;

	numlist=numargs;
	current=currentindex;
	copydata= new char*[numargs];

	for (int j=0; j<numargs; j++)
		copydata[j]=argdata[j];

	droplist.SetList(numargs, argdata);
	SetCurrent(current);
}

// -------------------------------------

	